module dfc.layout.base;

import win32.api;
import win32.commdlg;
import dfc.bitmap;
import dfc.core;
import dfc.button;
import dfc.tree;
import dfc.edit;
import winex.winmm;

interface HasPreferedSize {
	void getPreferedSize( out int width, out int height );
}

interface HasNotifyOnSize {
	void newSizeSet( int width, int height );
}

public class LayoutItem {
	wchar[] constraint;
	this( wchar[] cst ) { constraint = cst; }
	void reshape( int x, int y, int w, int h ) { }
	void getSize( out int w, out int h ) { }
	void enable( bit enabled ) { }
}

public class WndLayout : LayoutItem {
	DWnd parent;
	this( DWnd wnd, wchar[] cst ) { super( cst ); parent = wnd; }
	void reshape( int x, int y, int w, int h ) {
		parent.moveWindow( x, y, w, h, true );
		HasNotifyOnSize sizer;
		sizer = cast(HasNotifyOnSize)parent;
		if ( sizer ) {
			sizer.newSizeSet( w, h );
		}
	}
	void getSize( out int w, out int h ) {
		HasPreferedSize sizer;
		sizer = cast(HasPreferedSize)parent;
		if ( sizer ) {
			sizer.getPreferedSize( w, h );
		} else {
			RECT r;
			parent.getWindowBounds( r );
			w = r.right;
			h = r.bottom;
		}
	}
	void enable( bit enabled ) { parent.showWindow( enabled ); }
}

public class NestedLayout : LayoutItem {
	LayoutManager parent;
	int width, height;
	this( LayoutManager mgr, wchar[] cst ) { super( cst ); parent = mgr; }
	void reshape( int x, int y, int w, int h ) {
		parent.resize( x, y, w, h );
		width  = w;
		height = h;
	}
	void getSize( out int w, out int h ) {
		w = width;
		h = height;
	}
}

public class LayoutManager {

	public this() { }

	public final void add( DWnd child, wchar[] constraints ) {
		addi( new WndLayout( child, constraints ) );
	}

	public final void add( LayoutManager child, wchar[] constraints ) {
		addi( new NestedLayout( child, constraints ) );
	}

	public final void add( DWnd child, wchar[] constraints, int w, int h ) {
		WndLayout wl;
		addi( wl = new WndLayout( child, constraints ) );
		wl.reshape( 0,0,w, h );
	}

	public final void add( LayoutManager child, wchar[] constraints, int w, int h ) {
		NestedLayout nl;
		addi( nl = new NestedLayout( child, constraints ) );
		nl.reshape(0,0, w, h );
	}

	protected void addi( LayoutItem item ) {
		throw new Exception( "LayoutManager::add(LayoutItem) must be defined" );
	}

	public void resize( int x, int y, int w, int h ) {
		throw new Exception( "LayoutManager::resize(int,int,int,int) must be defined" );
	}

	static void connect( DWnd wnd, LayoutManager mgr ) {
		WndLayoutController.connect( wnd, mgr );
	}
}

public class WndLayoutController {
	protected DWnd parent;
	protected LayoutManager manager;
	this( DWnd p, LayoutManager mgr ) {
		parent  = p;
		manager = mgr;
		p.addHandler( WM_SIZE, new ResizeMessageSlot( &onResize ) );
	}

	private final LRESULT onResize( int width, int height ) {
		manager.resize( 0, 0, width, height );
		return cast(LRESULT)0;
	}

	static void connect( DWnd wnd, LayoutManager mgr ) {
		new WndLayoutController( wnd, mgr );
	}
}
